@@ -1,7 +1,4 @@
-{-# LANGUAGE BangPatterns          #-}
 {-# LANGUAGE QuantifiedConstraints #-}
-{-# LANGUAGE TypeFamilies          #-}
-{-# LANGUAGE TypeOperators         #-}
 
 -- | 'io-sim' implementation of 'TQueue', 'TBQueue' and 'MVar'.
 --
@@ -11,18 +8,17 @@
 --
 module Control.Monad.IOSim.STM where
 
-import           Control.Exception (SomeAsyncException (..))
+import Control.Exception (SomeAsyncException (..))
 
-import           Control.Concurrent.Class.MonadSTM.TVar
-import           Control.Monad.Class.MonadSTM (MonadInspectSTM (..),
-                     MonadLabelledSTM, MonadSTM (..), MonadTraceSTM,
-                     TraceValue (..))
-import           Control.Monad.Class.MonadThrow
+import Control.Concurrent.Class.MonadSTM.TVar
+import Control.Monad.Class.MonadSTM (MonadInspectSTM (..), MonadLabelledSTM,
+           MonadSTM (..), MonadTraceSTM, TraceValue (..))
+import Control.Monad.Class.MonadThrow
 
-import           Numeric.Natural (Natural)
+import Numeric.Natural (Natural)
 
-import           Data.Deque.Strict (Deque)
-import qualified Data.Deque.Strict as Deque
+import Data.Deque.Strict (Deque)
+import Data.Deque.Strict qualified as Deque
 
 --
 -- Default TQueue implementation in terms of a 'TVar' (used by sim)
@@ -39,7 +35,7 @@ traceTQueueDefault
   :: MonadTraceSTM m
   => proxy m
   -> TQueueDefault m a
-  -> (Maybe [a] -> [a] -> InspectMonad m TraceValue)
+  -> (Maybe [a] -> [a] -> InspectMonadSTM m TraceValue)
   -> STM m ()
 traceTQueueDefault p (TQueue queue) f =
     traceTVar p queue
@@ -96,7 +92,10 @@ tryPeekTQueueDefault (TQueue queue) = do
       []   -> Nothing
 
 flushTQueueDefault :: MonadSTM m => TQueueDefault m a -> STM m [a]
-flushTQueueDefault (TQueue queue) = uncurry (++) <$> readTVar queue
+flushTQueueDefault (TQueue queue) = do
+  (xs, ys) <- readTVar queue
+  writeTVar queue ([], [])
+  pure (xs <> reverse ys)
 
 unGetTQueueDefault :: MonadSTM m => TQueueDefault m a -> a -> STM m ()
 unGetTQueueDefault (TQueue queue) a = do
@@ -120,7 +119,7 @@ traceTBQueueDefault
   :: MonadTraceSTM m
   => proxy m
   -> TBQueueDefault m a
-  -> (Maybe [a] -> [a] -> InspectMonad m TraceValue)
+  -> (Maybe [a] -> [a] -> InspectMonadSTM m TraceValue)
   -> STM m ()
 traceTBQueueDefault p (TBQueue queue _size) f =
     traceTVar p queue (\mas as -> f (g <$> mas) (g as))
@@ -147,12 +146,8 @@ tryReadTBQueueDefault (TBQueue queue _size) = do
       return (Just x)
     [] ->
       case reverse ys of
-        [] -> do
-          writeTVar queue $! (xs, r', ys, w)
-          return Nothing
+        [] -> return Nothing
 
-        -- NB. lazy: we want the transaction to be
-        -- short, otherwise it will conflict
         (z:zs) -> do
            writeTVar queue $! (zs, r', [], w)
            return (Just z)
@@ -232,7 +227,7 @@ unGetTBQueueDefault (TBQueue queue _size) a = do
 --
 -- /Implementation details:/
 --
--- 'STM' does not guarantee fairness, instead it provide compositionally.
+-- @STM@ does not guarantee fairness, instead it provides compositionally.
 -- Fairness of 'putMVarDefault' and 'takeMVarDefault' is provided by tracking
 -- queue of blocked operation in the 'MVarState', e.g.  when a 'putMVarDefault'
 -- is scheduled on a full 'MVar', the request is put on to the back of the queue
@@ -259,13 +254,17 @@ data MVarState m a = MVarEmpty   !(Deque (TVar m (Maybe a))) -- blocked on take
 newEmptyMVarDefault :: MonadSTM m => m (MVarDefault m a)
 newEmptyMVarDefault = MVar <$> newTVarIO (MVarEmpty mempty mempty)
 
+labelMVarDefault
+  :: MonadLabelledSTM m
+  => MVarDefault m a -> String -> m ()
+labelMVarDefault (MVar tvar) = atomically . labelTVar tvar
 
 newMVarDefault :: MonadSTM m => a -> m (MVarDefault m a)
 newMVarDefault a = MVar <$> newTVarIO (MVarFull a mempty)
 
 
 putMVarDefault :: ( MonadMask  m
-                  , MonadSTM   m
+                  , MonadLabelledSTM m
                   , forall x tvar. tvar ~ TVar m x => Eq tvar
                   )
                => MVarDefault m a -> a -> m ()
@@ -276,6 +275,7 @@ putMVarDefault (MVar tv) x = mask_ $ do
         -- It's full, add ourselves to the end of the 'put' blocked queue.
         MVarFull x' putq -> do
           putvar <- newTVar False
+          labelTVar putvar "internal-putvar"
           writeTVar tv (MVarFull x' (Deque.snoc (x, putvar) putq))
           return (Just putvar)
 
@@ -348,7 +348,7 @@ tryPutMVarDefault (MVar tv) x =
 
 
 takeMVarDefault :: ( MonadMask m
-                   , MonadSTM  m
+                   , MonadLabelledSTM m
                    , forall x tvar. tvar ~ TVar m x => Eq tvar
                    )
                 => MVarDefault m a
@@ -360,6 +360,7 @@ takeMVarDefault (MVar tv) = mask_ $ do
         -- It's empty, add ourselves to the end of the 'take' blocked queue.
         MVarEmpty takeq readq -> do
           takevar <- newTVar Nothing
+          labelTVar takevar "internal-takevar"
           writeTVar tv (MVarEmpty (Deque.snoc takevar takeq) readq)
           return (Left takevar)
 
@@ -390,7 +391,24 @@ takeMVarDefault (MVar tv) = mask_ $ do
                 -- takevar; we need to remove it from 'takeq', otherwise we
                 -- will have a space leak.
                 let takeq' = Deque.filter (/= takevar) takeq
-                writeTVar tv (MVarEmpty takeq' readq)
+                takevalue <- readTVar takevar
+                case takevalue of
+                  Nothing ->
+                    writeTVar tv (MVarEmpty takeq' readq)
+                  -- we were given a value before we could read it. Relay it to any
+                  -- new reading threads and possible the next take thread.
+                  Just x -> do
+                    -- notify readers
+                    mapM_ (\readvar -> writeTVar readvar (Just x)) readq
+
+                    -- notify first `takeMVar` thread
+                    case Deque.uncons takeq' of
+                      Nothing ->
+                        writeTVar tv (MVarFull x mempty)
+
+                      Just (takevar', takeq'') -> do
+                        writeTVar takevar' (Just x)
+                        writeTVar tv (MVarEmpty takeq'' mempty)
 
               -- This case is unlikely but possible if another thread ran
               -- first and modified the mvar. This situation is fine as far as
@@ -431,7 +449,7 @@ tryTakeMVarDefault (MVar tv) = do
 -- 'putMVar' value.  It will also not block if the 'MVar' is full, even if there
 -- are other threads attempting to 'putMVar'.
 --
-readMVarDefault :: ( MonadSTM m
+readMVarDefault :: ( MonadLabelledSTM m
                    , MonadMask m
                    , forall x tvar. tvar ~ TVar m x => Eq tvar
                    )
@@ -444,6 +462,7 @@ readMVarDefault (MVar tv) = do
         -- It's empty, add ourselves to the 'read' blocked queue.
         MVarEmpty takeq readq -> do
           readvar <- newTVar Nothing
+          labelTVar readvar "internal-readvar"
           writeTVar tv (MVarEmpty takeq (Deque.snoc readvar readq))
           return (Left readvar)
 
